@@ -169,7 +169,7 @@ def DownloadData(Dir, Var, Startdate, Enddate, latlim, lonlim, TimeStep, Period,
                     url_start = r"https://opendap.nccs.nasa.gov/dods/GEOS-5/MERRAero/hourly/tavg3hr_2d_asm_Nx."
                 
                 url_MERRA = url_start + 'ascii?%s[%s:1:%s][%s:1:%s][%s:1:%s]' %(Var, IDz_start,IDz_end, int(IDy[0]),int(IDy[1]),int(IDx[0]),int(IDx[1]))
-            
+                
             if TimeStep == "yearly":
                 url_start = r"https://opendap.nccs.nasa.gov/dods/GEOS-5/MERRAero/monthly/tavg3hr_2d_asm_Nx."
                 url_MERRA = url_start + 'ascii?%s[%s:1:%s][%s:1:%s][%s:1:%s]' %(Var, IDz_start,IDz_end, int(IDy[0]),int(IDy[1]),int(IDx[0]),int(IDx[1]))
@@ -344,7 +344,7 @@ def Get_NC_data_end(file_name,Var, TimeStep, Period, IDy, IDx, VarInfo):
             
         data[data==-9999] = np.nan
         data_min = np.nanmin(data, 0)
-        data_max = np.nanmin(data, 0)     
+        data_max = np.nanmax(data, 0)     
         
     return(data_end, data_min, data_max)
 
